@@ -24,7 +24,7 @@
  */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif /* defined(HAVE_CONFIG_H) */
 
 #include <assert.h>
 
@@ -39,9 +39,9 @@
 #define PRINTF_DEBUG 0
 #if PRINTF_DEBUG
 #  define DEBUG_MSG(...) fprintf(stderr, __VA_ARGS__)
-#else
+#else /* !PRINTF_DEBUG */
 #  define DEBUG_MSG(...) (void)0
-#endif
+#endif /* !PRINTF_DEBUG */
 
 ngtcp2_crypto_aead *ngtcp2_crypto_aead_aes_128_gcm(ngtcp2_crypto_aead *aead) {
   return ngtcp2_crypto_aead_init(aead, (void *)wolfSSL_EVP_aes_128_gcm());
@@ -65,7 +65,7 @@ ngtcp2_crypto_aead *ngtcp2_crypto_aead_init(ngtcp2_crypto_aead *aead,
                                             void *aead_native_handle) {
   aead->native_handle = aead_native_handle;
   aead->max_overhead = wolfSSL_quic_get_aead_tag_len(
-      (const WOLFSSL_EVP_CIPHER *)(aead_native_handle));
+    (const WOLFSSL_EVP_CIPHER *)(aead_native_handle));
   return aead;
 }
 
@@ -73,9 +73,8 @@ ngtcp2_crypto_aead *ngtcp2_crypto_aead_retry(ngtcp2_crypto_aead *aead) {
   return ngtcp2_crypto_aead_init(aead, (void *)wolfSSL_EVP_aes_128_gcm());
 }
 
-static uint64_t crypto_wolfssl_get_aead_max_encryption(WOLFSSL *ssl) {
-  const WOLFSSL_EVP_CIPHER *aead = wolfSSL_quic_get_aead(ssl);
-
+static uint64_t
+crypto_aead_get_aead_max_encryption(const WOLFSSL_EVP_CIPHER *aead) {
   if (wolfSSL_quic_aead_is_gcm(aead)) {
     return NGTCP2_CRYPTO_MAX_ENCRYPTION_AES_GCM;
   }
@@ -88,9 +87,8 @@ static uint64_t crypto_wolfssl_get_aead_max_encryption(WOLFSSL *ssl) {
   return 0;
 }
 
-static uint64_t crypto_wolfssl_get_aead_max_decryption_failure(WOLFSSL *ssl) {
-  const WOLFSSL_EVP_CIPHER *aead = wolfSSL_quic_get_aead(ssl);
-
+static uint64_t
+crypto_aead_get_aead_max_decryption_failure(const WOLFSSL_EVP_CIPHER *aead) {
   if (wolfSSL_quic_aead_is_gcm(aead)) {
     return NGTCP2_CRYPTO_MAX_DECRYPTION_FAILURE_AES_GCM;
   }
@@ -103,16 +101,30 @@ static uint64_t crypto_wolfssl_get_aead_max_decryption_failure(WOLFSSL *ssl) {
   return 0;
 }
 
+static int supported_aead(const WOLFSSL_EVP_CIPHER *aead) {
+  return wolfSSL_quic_aead_is_gcm(aead) ||
+         wolfSSL_quic_aead_is_chacha20(aead) || wolfSSL_quic_aead_is_ccm(aead);
+}
+
 ngtcp2_crypto_ctx *ngtcp2_crypto_ctx_tls(ngtcp2_crypto_ctx *ctx,
                                          void *tls_native_handle) {
   WOLFSSL *ssl = tls_native_handle;
+  const WOLFSSL_EVP_CIPHER *aead = wolfSSL_quic_get_aead(ssl);
 
-  ngtcp2_crypto_aead_init(&ctx->aead, (void *)wolfSSL_quic_get_aead(ssl));
+  if (aead == NULL) {
+    return NULL;
+  }
+
+  if (!supported_aead(aead)) {
+    return NULL;
+  }
+
+  ngtcp2_crypto_aead_init(&ctx->aead, (void *)aead);
   ctx->md.native_handle = (void *)wolfSSL_quic_get_md(ssl);
   ctx->hp.native_handle = (void *)wolfSSL_quic_get_hp(ssl);
-  ctx->max_encryption = crypto_wolfssl_get_aead_max_encryption(ssl);
+  ctx->max_encryption = crypto_aead_get_aead_max_encryption(aead);
   ctx->max_decryption_failure =
-      crypto_wolfssl_get_aead_max_decryption_failure(ssl);
+    crypto_aead_get_aead_max_decryption_failure(aead);
   return ctx;
 }
 
@@ -191,7 +203,7 @@ int ngtcp2_crypto_cipher_ctx_encrypt_init(ngtcp2_crypto_cipher_ctx *cipher_ctx,
   WOLFSSL_EVP_CIPHER_CTX *actx;
 
   actx =
-      wolfSSL_quic_crypt_new(cipher->native_handle, key, NULL, /* encrypt */ 1);
+    wolfSSL_quic_crypt_new(cipher->native_handle, key, NULL, /* encrypt */ 1);
   if (actx == NULL) {
     return -1;
   }
@@ -267,7 +279,6 @@ int ngtcp2_crypto_decrypt(uint8_t *dest, const ngtcp2_crypto_aead *aead,
   if (wolfSSL_quic_aead_decrypt(dest, aead_ctx->native_handle, ciphertext,
                                 ciphertextlen, nonce, aad,
                                 aadlen) != WOLFSSL_SUCCESS) {
-
     DEBUG_MSG("WOLFSSL: decrypt FAILED\n");
     return -1;
   }
@@ -284,11 +295,11 @@ int ngtcp2_crypto_hp_mask(uint8_t *dest, const ngtcp2_crypto_cipher *hp,
   (void)hp;
 
   if (wolfSSL_EVP_EncryptInit_ex(actx, NULL, NULL, NULL, sample) !=
-          WOLFSSL_SUCCESS ||
+        WOLFSSL_SUCCESS ||
       wolfSSL_EVP_CipherUpdate(actx, dest, &len, PLAINTEXT,
                                sizeof(PLAINTEXT) - 1) != WOLFSSL_SUCCESS ||
       wolfSSL_EVP_EncryptFinal_ex(actx, dest + sizeof(PLAINTEXT) - 1, &len) !=
-          WOLFSSL_SUCCESS) {
+        WOLFSSL_SUCCESS) {
     DEBUG_MSG("WOLFSSL: hp_mask FAILED\n");
     return -1;
   }
@@ -296,12 +307,12 @@ int ngtcp2_crypto_hp_mask(uint8_t *dest, const ngtcp2_crypto_cipher *hp,
   return 0;
 }
 
-int ngtcp2_crypto_read_write_crypto_data(ngtcp2_conn *conn,
-                                         ngtcp2_crypto_level crypto_level,
-                                         const uint8_t *data, size_t datalen) {
+int ngtcp2_crypto_read_write_crypto_data(
+  ngtcp2_conn *conn, ngtcp2_encryption_level encryption_level,
+  const uint8_t *data, size_t datalen) {
   WOLFSSL *ssl = ngtcp2_conn_get_tls_native_handle(conn);
   WOLFSSL_ENCRYPTION_LEVEL level =
-      ngtcp2_crypto_wolfssl_from_ngtcp2_crypto_level(crypto_level);
+    ngtcp2_crypto_wolfssl_from_ngtcp2_encryption_level(encryption_level);
   int rv;
   int err;
 
@@ -332,7 +343,7 @@ int ngtcp2_crypto_read_write_crypto_data(ngtcp2_conn *conn,
     }
 
     DEBUG_MSG("WOLFSSL: handshake done\n");
-    ngtcp2_conn_handshake_completed(conn);
+    ngtcp2_conn_tls_handshake_completed(conn);
   }
 
   rv = wolfSSL_process_quic_post_handshake(ssl);
@@ -363,7 +374,7 @@ int ngtcp2_crypto_set_remote_transport_params(ngtcp2_conn *conn, void *tls) {
   wolfSSL_get_peer_quic_transport_params(ssl, &tp, &tplen);
   DEBUG_MSG("WOLFSSL: get peer transport params, len=%lu\n", tplen);
 
-  rv = ngtcp2_conn_decode_remote_transport_params(conn, tp, tplen);
+  rv = ngtcp2_conn_decode_and_set_remote_transport_params(conn, tp, tplen);
   if (rv != 0) {
     DEBUG_MSG("WOLFSSL: decode peer transport params failed, rv=%d\n", rv);
     ngtcp2_conn_set_tls_error(conn, rv);
@@ -384,17 +395,17 @@ int ngtcp2_crypto_set_local_transport_params(void *tls, const uint8_t *buf,
   return 0;
 }
 
-ngtcp2_crypto_level ngtcp2_crypto_wolfssl_from_wolfssl_encryption_level(
-    WOLFSSL_ENCRYPTION_LEVEL wolfssl_level) {
+ngtcp2_encryption_level ngtcp2_crypto_wolfssl_from_wolfssl_encryption_level(
+  WOLFSSL_ENCRYPTION_LEVEL wolfssl_level) {
   switch (wolfssl_level) {
   case wolfssl_encryption_initial:
-    return NGTCP2_CRYPTO_LEVEL_INITIAL;
+    return NGTCP2_ENCRYPTION_LEVEL_INITIAL;
   case wolfssl_encryption_early_data:
-    return NGTCP2_CRYPTO_LEVEL_EARLY;
+    return NGTCP2_ENCRYPTION_LEVEL_0RTT;
   case wolfssl_encryption_handshake:
-    return NGTCP2_CRYPTO_LEVEL_HANDSHAKE;
+    return NGTCP2_ENCRYPTION_LEVEL_HANDSHAKE;
   case wolfssl_encryption_application:
-    return NGTCP2_CRYPTO_LEVEL_APPLICATION;
+    return NGTCP2_ENCRYPTION_LEVEL_1RTT;
   default:
     assert(0);
     abort(); /* if NDEBUG is set */
@@ -402,16 +413,16 @@ ngtcp2_crypto_level ngtcp2_crypto_wolfssl_from_wolfssl_encryption_level(
 }
 
 WOLFSSL_ENCRYPTION_LEVEL
-ngtcp2_crypto_wolfssl_from_ngtcp2_crypto_level(
-    ngtcp2_crypto_level crypto_level) {
-  switch (crypto_level) {
-  case NGTCP2_CRYPTO_LEVEL_INITIAL:
+ngtcp2_crypto_wolfssl_from_ngtcp2_encryption_level(
+  ngtcp2_encryption_level encryption_level) {
+  switch (encryption_level) {
+  case NGTCP2_ENCRYPTION_LEVEL_INITIAL:
     return wolfssl_encryption_initial;
-  case NGTCP2_CRYPTO_LEVEL_HANDSHAKE:
+  case NGTCP2_ENCRYPTION_LEVEL_HANDSHAKE:
     return wolfssl_encryption_handshake;
-  case NGTCP2_CRYPTO_LEVEL_APPLICATION:
+  case NGTCP2_ENCRYPTION_LEVEL_1RTT:
     return wolfssl_encryption_application;
-  case NGTCP2_CRYPTO_LEVEL_EARLY:
+  case NGTCP2_ENCRYPTION_LEVEL_0RTT:
     return wolfssl_encryption_early_data;
   default:
     assert(0);
@@ -445,8 +456,8 @@ static int set_encryption_secrets(WOLFSSL *ssl,
                                   const uint8_t *tx_secret, size_t secretlen) {
   ngtcp2_crypto_conn_ref *conn_ref = SSL_get_app_data(ssl);
   ngtcp2_conn *conn = conn_ref->get_conn(conn_ref);
-  ngtcp2_crypto_level level =
-      ngtcp2_crypto_wolfssl_from_wolfssl_encryption_level(wolfssl_level);
+  ngtcp2_encryption_level level =
+    ngtcp2_crypto_wolfssl_from_wolfssl_encryption_level(wolfssl_level);
 
   DEBUG_MSG("WOLFSSL: set encryption secrets, level=%d, rxlen=%lu, txlen=%lu\n",
             wolfssl_level, rx_secret ? secretlen : 0,
@@ -471,8 +482,8 @@ static int add_handshake_data(WOLFSSL *ssl,
                               const uint8_t *data, size_t datalen) {
   ngtcp2_crypto_conn_ref *conn_ref = SSL_get_app_data(ssl);
   ngtcp2_conn *conn = conn_ref->get_conn(conn_ref);
-  ngtcp2_crypto_level level =
-      ngtcp2_crypto_wolfssl_from_wolfssl_encryption_level(wolfssl_level);
+  ngtcp2_encryption_level level =
+    ngtcp2_crypto_wolfssl_from_wolfssl_encryption_level(wolfssl_level);
   int rv;
 
   DEBUG_MSG("WOLFSSL: add handshake data, level=%d len=%lu\n", wolfssl_level,
@@ -504,10 +515,10 @@ static int send_alert(WOLFSSL *ssl, enum wolfssl_encryption_level_t level,
 }
 
 static WOLFSSL_QUIC_METHOD quic_method = {
-    set_encryption_secrets,
-    add_handshake_data,
-    flush_flight,
-    send_alert,
+  set_encryption_secrets,
+  add_handshake_data,
+  flush_flight,
+  send_alert,
 };
 
 static void crypto_wolfssl_configure_context(WOLFSSL_CTX *ssl_ctx) {
@@ -520,7 +531,7 @@ int ngtcp2_crypto_wolfssl_configure_server_context(WOLFSSL_CTX *ssl_ctx) {
   crypto_wolfssl_configure_context(ssl_ctx);
 #if PRINTF_DEBUG
   wolfSSL_Debugging_ON();
-#endif
+#endif /* PRINTF_DEBUG */
   return 0;
 }
 
@@ -529,6 +540,6 @@ int ngtcp2_crypto_wolfssl_configure_client_context(WOLFSSL_CTX *ssl_ctx) {
   wolfSSL_CTX_UseSessionTicket(ssl_ctx);
 #if PRINTF_DEBUG
   wolfSSL_Debugging_ON();
-#endif
+#endif /* PRINTF_DEBUG */
   return 0;
 }
