@@ -31,7 +31,7 @@
 
 #include <vector>
 #include <deque>
-#include <map>
+#include <unordered_map>
 #include <string_view>
 #include <memory>
 #include <set>
@@ -107,6 +107,8 @@ public:
   std::pair<std::span<const uint8_t>, int>
   send_packet(const Endpoint &ep, const ngtcp2_addr &remote_addr,
               unsigned int ecn, std::span<const uint8_t> data, size_t gso_size);
+  int send_packet_or_blocked(const ngtcp2_path &path, unsigned int ecn,
+                             std::span<const uint8_t> data, size_t gso_size);
   int on_stream_close(int64_t stream_id, uint64_t app_error_code);
   int on_extend_max_streams();
   int handle_error();
@@ -138,11 +140,12 @@ public:
 
   void write_qlog(const void *data, size_t datalen);
 
-  void on_send_blocked(const Endpoint &ep, const ngtcp2_addr &remote_addr,
-                       unsigned int ecn, std::span<const uint8_t> data,
-                       size_t gso_size);
+  void on_send_blocked(const ngtcp2_path &path, unsigned int ecn,
+                       std::span<const uint8_t> data, size_t gso_size);
   void start_wev_endpoint(const Endpoint &ep);
   int send_blocked_packet();
+  ngtcp2_ssize write_pkt(ngtcp2_path *path, ngtcp2_pkt_info *pi, uint8_t *dest,
+                         size_t destlen, ngtcp2_tstamp ts);
 
   const std::vector<uint32_t> &get_offered_versions() const;
 
@@ -160,7 +163,7 @@ private:
   ev_timer delay_stream_timer_;
   ev_signal sigintev_;
   struct ev_loop *loop_;
-  std::map<int64_t, std::unique_ptr<Stream>> streams_;
+  std::unordered_map<int64_t, std::unique_ptr<Stream>> streams_;
   std::set<Stream *, StreamIDLess> sendq_;
   std::vector<uint32_t> offered_versions_;
   // addr_ is the server host address.
@@ -184,8 +187,6 @@ private:
 
   struct {
     bool send_blocked;
-    size_t num_blocked;
-    size_t num_blocked_sent;
     // blocked field is effective only when send_blocked is true.
     struct {
       const Endpoint *endpoint;
@@ -193,7 +194,7 @@ private:
       unsigned int ecn;
       std::span<const uint8_t> data;
       size_t gso_size;
-    } blocked[2];
+    } blocked;
     std::array<uint8_t, 64_k> data;
   } tx_;
 };
