@@ -346,7 +346,7 @@ Handler::Handler(struct ev_loop *loop, Server *server)
     nkey_update_(0),
     no_gso_{
 #ifdef UDP_SEGMENT
-      false
+      config.no_gso
 #else  // !defined(UDP_SEGMENT)
       true
 #endif // !defined(UDP_SEGMENT)
@@ -355,7 +355,7 @@ Handler::Handler(struct ev_loop *loop, Server *server)
       .next_pkts_recv = 1,
     },
     tx_{
-      .data = std::unique_ptr<uint8_t[]>(new uint8_t[NGTCP2_TX_BUFLEN]),
+      .data = std::make_unique_for_overwrite<uint8_t[]>(NGTCP2_TX_BUFLEN),
     } {
   ev_io_init(&wev_, writecb, 0, EV_WRITE);
   wev_.data = this;
@@ -515,9 +515,9 @@ void Handler::on_stream_open(int64_t stream_id) {
   if (!ngtcp2_is_bidi_stream(stream_id)) {
     return;
   }
-  auto it = streams_.find(stream_id);
-  (void)it;
-  assert(it == std::ranges::end(streams_));
+
+  assert(!streams_.contains(stream_id));
+
   streams_.emplace(stream_id, std::make_unique<Stream>(stream_id, this));
 }
 
@@ -2776,12 +2776,15 @@ Options:
               Specify UDP datagram payload sizes  to probe in Path MTU
               Discovery.  <SIZE> must be strictly larger than 1200.
   --ech-config-file=<PATH>
-              Read private  key and  ECHConfig from |PATH|.   The file
-              denoted by |PATH| must contain private key and ECHConfig
+              Read private  key and  ECHConfig from <PATH>.   The file
+              denoted by <PATH> must contain private key and ECHConfig
               as                      described                     in
               https://datatracker.ietf.org/doc/html/draft-farrell-tls-pemesni.
               ECH configuration  is only applied if  an underlying TLS
               stack supports it.
+  --no-gso    Disables GSO.
+  --show-stat Print the connection statistics when the connection is
+              closed.
   -h, --help  Display this help and exit.
 
 ---
@@ -2854,6 +2857,8 @@ int main(int argc, char **argv) {
       {"initial-pkt-num", required_argument, &flag, 31},
       {"pmtud-probes", required_argument, &flag, 32},
       {"ech-config-file", required_argument, &flag, 33},
+      {"no-gso", no_argument, &flag, 35},
+      {"show-stat", no_argument, &flag, 36},
       {},
     };
 
@@ -3207,6 +3212,14 @@ int main(int argc, char **argv) {
         // --ech-config-file
         ech_config_file = optarg;
         break;
+      case 35:
+        // --no-gso
+        config.no_gso = true;
+        break;
+      case 36:
+        // --show-stat
+        config.show_stat = true;
+        break;
       }
       break;
     default:
