@@ -27,10 +27,12 @@
 #include <cstring>
 #include <iostream>
 #include <fstream>
+#include <algorithm>
 
 #include <ngtcp2/ngtcp2_crypto_boringssl.h>
 
 #include <openssl/err.h>
+#include <openssl/hpke.h>
 
 #include "server_base.h"
 #include "template.h"
@@ -54,8 +56,6 @@ int alpn_select_proto_h3_cb(SSL *ssl, const unsigned char **out,
                             unsigned int inlen, void *arg) {
   auto conn_ref = static_cast<ngtcp2_crypto_conn_ref *>(SSL_get_app_data(ssl));
   auto h = static_cast<HandlerBase *>(conn_ref->user_data);
-  const uint8_t *alpn;
-  size_t alpnlen;
   // This should be the negotiated version, but we have not set the
   // negotiated version when this callback is called.
   auto version = ngtcp2_conn_get_client_chosen_version(h->conn());
@@ -63,8 +63,6 @@ int alpn_select_proto_h3_cb(SSL *ssl, const unsigned char **out,
   switch (version) {
   case NGTCP2_PROTO_VER_V1:
   case NGTCP2_PROTO_VER_V2:
-    alpn = H3_ALPN_V1;
-    alpnlen = str_size(H3_ALPN_V1);
     break;
   default:
     if (!config.quiet) {
@@ -74,16 +72,17 @@ int alpn_select_proto_h3_cb(SSL *ssl, const unsigned char **out,
     return SSL_TLSEXT_ERR_ALERT_FATAL;
   }
 
-  for (auto p = in, end = in + inlen; p + alpnlen <= end; p += *p + 1) {
-    if (std::equal(alpn, alpn + alpnlen, p)) {
-      *out = p + 1;
-      *outlen = *p;
+  for (auto s = std::span{in, inlen}; s.size() >= H3_ALPN_V1.size();
+       s = s.subspan(s[0] + 1)) {
+    if (std::ranges::equal(H3_ALPN_V1, s.first(H3_ALPN_V1.size()))) {
+      *out = &s[1];
+      *outlen = s[0];
       return SSL_TLSEXT_ERR_OK;
     }
   }
 
   if (!config.quiet) {
-    std::cerr << "Client did not present ALPN " << &alpn[1] << std::endl;
+    std::cerr << "Client did not present ALPN " << &H3_ALPN_V1[1] << std::endl;
   }
 
   return SSL_TLSEXT_ERR_ALERT_FATAL;
@@ -96,8 +95,6 @@ int alpn_select_proto_hq_cb(SSL *ssl, const unsigned char **out,
                             unsigned int inlen, void *arg) {
   auto conn_ref = static_cast<ngtcp2_crypto_conn_ref *>(SSL_get_app_data(ssl));
   auto h = static_cast<HandlerBase *>(conn_ref->user_data);
-  const uint8_t *alpn;
-  size_t alpnlen;
   // This should be the negotiated version, but we have not set the
   // negotiated version when this callback is called.
   auto version = ngtcp2_conn_get_client_chosen_version(h->conn());
@@ -105,8 +102,6 @@ int alpn_select_proto_hq_cb(SSL *ssl, const unsigned char **out,
   switch (version) {
   case NGTCP2_PROTO_VER_V1:
   case NGTCP2_PROTO_VER_V2:
-    alpn = HQ_ALPN_V1;
-    alpnlen = str_size(HQ_ALPN_V1);
     break;
   default:
     if (!config.quiet) {
@@ -116,16 +111,17 @@ int alpn_select_proto_hq_cb(SSL *ssl, const unsigned char **out,
     return SSL_TLSEXT_ERR_ALERT_FATAL;
   }
 
-  for (auto p = in, end = in + inlen; p + alpnlen <= end; p += *p + 1) {
-    if (std::equal(alpn, alpn + alpnlen, p)) {
-      *out = p + 1;
-      *outlen = *p;
+  for (auto s = std::span{in, inlen}; s.size() >= HQ_ALPN_V1.size();
+       s = s.subspan(s[0] + 1)) {
+    if (std::ranges::equal(HQ_ALPN_V1, s.first(HQ_ALPN_V1.size()))) {
+      *out = &s[1];
+      *outlen = s[0];
       return SSL_TLSEXT_ERR_OK;
     }
   }
 
   if (!config.quiet) {
-    std::cerr << "Client did not present ALPN " << &alpn[1] << std::endl;
+    std::cerr << "Client did not present ALPN " << &HQ_ALPN_V1[1] << std::endl;
   }
 
   return SSL_TLSEXT_ERR_ALERT_FATAL;
@@ -218,6 +214,39 @@ int TLSServerContext::init(const char *private_key_file, const char *cert_file,
   }
 #endif // defined(HAVE_LIBBROTLI)
 
+  if (!config.ech_config.ech_config.empty()) {
+    const auto &echconf = config.ech_config;
+
+    auto pkey = EVP_HPKE_KEY_new();
+
+    if (EVP_HPKE_KEY_init(pkey, EVP_hpke_x25519_hkdf_sha256(),
+                          echconf.private_key.bytes.data(),
+                          echconf.private_key.bytes.size()) != 1) {
+      std::cerr << "EVP_HPKE_KEY_init failed: "
+                << ERR_error_string(ERR_get_error(), nullptr) << std::endl;
+
+      return -1;
+    }
+
+    auto pkey_d = defer(EVP_HPKE_KEY_free, pkey);
+
+    auto keys = SSL_ECH_KEYS_new();
+    auto keys_d = defer(SSL_ECH_KEYS_free, keys);
+
+    if (SSL_ECH_KEYS_add(keys, 1, echconf.ech_config.data(),
+                         echconf.ech_config.size(), pkey) != 1) {
+      std::cerr << "SSL_ECH_KEYS_add failed: "
+                << ERR_error_string(ERR_get_error(), nullptr) << std::endl;
+      return -1;
+    }
+
+    if (SSL_CTX_set1_ech_keys(ssl_ctx_, keys) != 1) {
+      std::cerr << "SSL_CTX_set1_ech_keys failed: "
+                << ERR_error_string(ERR_get_error(), nullptr) << std::endl;
+      return -1;
+    }
+  }
+
   return 0;
 }
 
@@ -225,7 +254,7 @@ extern std::ofstream keylog_file;
 
 namespace {
 void keylog_callback(const SSL *ssl, const char *line) {
-  keylog_file.write(line, strlen(line));
+  keylog_file.write(line, static_cast<std::streamsize>(strlen(line)));
   keylog_file.put('\n');
   keylog_file.flush();
 }
