@@ -48,8 +48,8 @@ int on_client_hello_h3_cb(ptls_on_client_hello_t *self, ptls_t *ptls,
 
   for (size_t i = 0; i < negprotos.count; ++i) {
     auto &proto = negprotos.list[i];
-    if (H3_ALPN_V1[0] == proto.len &&
-        memcmp(&H3_ALPN_V1[1], proto.base, proto.len) == 0) {
+    if (std::ranges::equal(H3_ALPN_V1.subspan(1),
+                           std::span{proto.base, proto.len})) {
       if (ptls_set_negotiated_protocol(
             ptls, reinterpret_cast<char *>(proto.base), proto.len) != 0) {
         return -1;
@@ -72,8 +72,8 @@ int on_client_hello_hq_cb(ptls_on_client_hello_t *self, ptls_t *ptls,
 
   for (size_t i = 0; i < negprotos.count; ++i) {
     auto &proto = negprotos.list[i];
-    if (HQ_ALPN_V1[0] == proto.len &&
-        memcmp(&HQ_ALPN_V1[1], proto.base, proto.len) == 0) {
+    if (std::ranges::equal(HQ_ALPN_V1.subspan(1),
+                           std::span{proto.base, proto.len})) {
       if (ptls_set_negotiated_protocol(
             ptls, reinterpret_cast<char *>(proto.base), proto.len) != 0) {
         return -1;
@@ -92,25 +92,21 @@ ptls_on_client_hello_t on_client_hello_hq = {on_client_hello_hq_cb};
 namespace {
 auto ticket_hmac = EVP_sha256();
 
-template <size_t N> void random_bytes(std::array<uint8_t, N> &dest) {
-  ptls_openssl_random_bytes(dest.data(), dest.size());
-}
-
-const std::array<uint8_t, 16> &get_ticket_key_name() {
+std::span<const uint8_t> get_ticket_key_name() {
   static std::array<uint8_t, 16> key_name;
-  random_bytes(key_name);
+  ptls_openssl_random_bytes(key_name.data(), key_name.size());
   return key_name;
 }
 
-const std::array<uint8_t, 32> &get_ticket_key() {
+std::span<const uint8_t> get_ticket_key() {
   static std::array<uint8_t, 32> key;
-  random_bytes(key);
+  ptls_openssl_random_bytes(key.data(), key.size());
   return key;
 }
 
-const std::array<uint8_t, 32> &get_ticket_hmac_key() {
+std::span<const uint8_t> get_ticket_hmac_key() {
   static std::array<uint8_t, 32> hmac_key;
-  random_bytes(hmac_key);
+  ptls_openssl_random_bytes(hmac_key.data(), hmac_key.size());
   return hmac_key;
 }
 } // namespace
@@ -124,14 +120,14 @@ int ticket_key_cb(unsigned char *key_name, unsigned char *iv,
                   HMAC_CTX *hctx,
 #endif // OPENSSL_VERSION_NUMBER < 0x30000000L
                   int enc) {
-  static const auto &static_key_name = get_ticket_key_name();
-  static const auto &static_key = get_ticket_key();
-  static const auto &static_hmac_key = get_ticket_hmac_key();
+  static const auto static_key_name = get_ticket_key_name();
+  static const auto static_key = get_ticket_key();
+  static const auto static_hmac_key = get_ticket_hmac_key();
 
   if (enc) {
     ptls_openssl_random_bytes(iv, EVP_MAX_IV_LENGTH);
 
-    memcpy(key_name, static_key_name.data(), static_key_name.size());
+    std::ranges::copy(static_key_name, key_name);
 
     if (!EVP_EncryptInit_ex(ctx, EVP_aes_256_cbc(), nullptr, static_key.data(),
                             iv)) {
@@ -152,8 +148,9 @@ int ticket_key_cb(unsigned char *key_name, unsigned char *iv,
       return 0;
     }
 #else  // OPENSSL_VERSION_NUMBER < 0x30000000L
-    if (!HMAC_Init_ex(hctx, static_hmac_key.data(), static_hmac_key.size(),
-                      ticket_hmac, nullptr)) {
+    if (!HMAC_Init_ex(hctx, static_hmac_key.data(),
+                      static_cast<int>(static_hmac_key.size()), ticket_hmac,
+                      nullptr)) {
       return 0;
     }
 #endif // OPENSSL_VERSION_NUMBER < 0x30000000L
@@ -161,7 +158,8 @@ int ticket_key_cb(unsigned char *key_name, unsigned char *iv,
     return 1;
   }
 
-  if (memcmp(key_name, static_key_name.data(), static_key_name.size()) != 0) {
+  if (!std::ranges::equal(std::span{key_name, static_key_name.size()},
+                          static_key_name)) {
     return 0;
   }
 
@@ -184,8 +182,9 @@ int ticket_key_cb(unsigned char *key_name, unsigned char *iv,
     return 0;
   }
 #else  // OPENSSL_VERSION_NUMBER < 0x30000000L
-  if (!HMAC_Init_ex(hctx, static_hmac_key.data(), static_hmac_key.size(),
-                    ticket_hmac, nullptr)) {
+  if (!HMAC_Init_ex(hctx, static_hmac_key.data(),
+                    static_cast<int>(static_hmac_key.size()), ticket_hmac,
+                    nullptr)) {
     return 0;
   }
 #endif // OPENSSL_VERSION_NUMBER < 0x30000000L
@@ -208,11 +207,12 @@ int encrypt_ticket_cb(ptls_encrypt_ticket_t *encrypt_ticket, ptls_t *ptls,
     // TODO Replace std::make_unique with
     // std::make_unique_for_overwrite when it is available.
     auto buf = std::make_unique<uint8_t[]>(src.len + sizeof(ver));
-    auto p = std::copy_n(src.base, src.len, buf.get());
-    p = std::copy_n(reinterpret_cast<uint8_t *>(&ver), sizeof(ver), p);
+    auto p = std::ranges::copy_n(src.base, as_signed(src.len), buf.get()).out;
+    p = std::ranges::copy_n(reinterpret_cast<uint8_t *>(&ver), sizeof(ver), p)
+          .out;
 
     src.base = buf.get();
-    src.len = p - buf.get();
+    src.len = as_unsigned(p - buf.get());
 
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
     rv = ptls_openssl_encrypt_ticket_evp(dst, src, ticket_key_cb);
@@ -258,8 +258,13 @@ ptls_key_exchange_algorithm_t *key_exchanges[] = {
 #if PTLS_OPENSSL_HAVE_X25519
   &ptls_openssl_x25519,
 #endif // PTLS_OPENSSL_X25519
-  &ptls_openssl_secp256r1, &ptls_openssl_secp384r1,
-  &ptls_openssl_secp521r1, nullptr,
+  &ptls_openssl_secp256r1,
+  &ptls_openssl_secp384r1,
+  &ptls_openssl_secp521r1,
+#if PTLS_OPENSSL_HAVE_X25519MLKEM768
+  &ptls_openssl_x25519mlkem768,
+#endif // PTLS_OPENSSL_HAVE_X25519MLKEM768
+  nullptr,
 };
 } // namespace
 
