@@ -32,47 +32,37 @@
 #include "ngtcp2_net.h"
 #include "ngtcp2_unreachable.h"
 
-const uint8_t *ngtcp2_get_uint64(uint64_t *dest, const uint8_t *p) {
-  uint64_t n;
-  memcpy(&n, p, sizeof(n));
-  *dest = ngtcp2_ntohl64(n);
-  return p + sizeof(n);
+const uint8_t *ngtcp2_get_uint64be(uint64_t *dest, const uint8_t *p) {
+  memcpy(dest, p, sizeof(*dest));
+  *dest = ngtcp2_ntohl64(*dest);
+  return p + sizeof(*dest);
 }
 
-const uint8_t *ngtcp2_get_uint48(uint64_t *dest, const uint8_t *p) {
-  uint64_t n = 0;
-  memcpy(((uint8_t *)&n) + 2, p, 6);
-  *dest = ngtcp2_ntohl64(n);
-  return p + 6;
+const uint8_t *ngtcp2_get_uint32be(uint32_t *dest, const uint8_t *p) {
+  memcpy(dest, p, sizeof(*dest));
+  *dest = ngtcp2_ntohl(*dest);
+  return p + sizeof(*dest);
 }
 
-const uint8_t *ngtcp2_get_uint32(uint32_t *dest, const uint8_t *p) {
-  uint32_t n;
-  memcpy(&n, p, sizeof(n));
-  *dest = ngtcp2_ntohl(n);
-  return p + sizeof(n);
-}
-
-const uint8_t *ngtcp2_get_uint24(uint32_t *dest, const uint8_t *p) {
-  uint32_t n = 0;
-  memcpy(((uint8_t *)&n) + 1, p, 3);
-  *dest = ngtcp2_ntohl(n);
+const uint8_t *ngtcp2_get_uint24be(uint32_t *dest, const uint8_t *p) {
+  *dest = 0;
+  memcpy(((uint8_t *)dest) + 1, p, 3);
+  *dest = ngtcp2_ntohl(*dest);
   return p + 3;
 }
 
-const uint8_t *ngtcp2_get_uint16(uint16_t *dest, const uint8_t *p) {
-  uint16_t n;
-  memcpy(&n, p, sizeof(n));
-  *dest = ngtcp2_ntohs(n);
-  return p + sizeof(n);
-}
-
 const uint8_t *ngtcp2_get_uint16be(uint16_t *dest, const uint8_t *p) {
   memcpy(dest, p, sizeof(*dest));
+  *dest = ngtcp2_ntohs(*dest);
   return p + sizeof(*dest);
 }
 
-static uint64_t get_uvarint(size_t *plen, const uint8_t *p) {
+const uint8_t *ngtcp2_get_uint16(uint16_t *dest, const uint8_t *p) {
+  memcpy(dest, p, sizeof(*dest));
+  return p + sizeof(*dest);
+}
+
+static const uint8_t *get_uvarint(uint64_t *dest, const uint8_t *p) {
   union {
     uint8_t n8;
     uint16_t n16;
@@ -80,42 +70,39 @@ static uint64_t get_uvarint(size_t *plen, const uint8_t *p) {
     uint64_t n64;
   } n;
 
-  *plen = (size_t)(1u << (*p >> 6));
-
-  switch (*plen) {
+  switch (*p >> 6) {
+  case 0:
+    *dest = *p++;
+    return p;
   case 1:
-    return *p;
-  case 2:
     memcpy(&n, p, 2);
     n.n8 &= 0x3f;
-    return ngtcp2_ntohs(n.n16);
-  case 4:
+    *dest = ngtcp2_ntohs(n.n16);
+
+    return p + 2;
+  case 2:
     memcpy(&n, p, 4);
     n.n8 &= 0x3f;
-    return ngtcp2_ntohl(n.n32);
-  case 8:
+    *dest = ngtcp2_ntohl(n.n32);
+
+    return p + 4;
+  case 3:
     memcpy(&n, p, 8);
     n.n8 &= 0x3f;
-    return ngtcp2_ntohl64(n.n64);
+    *dest = ngtcp2_ntohl64(n.n64);
+
+    return p + 8;
   default:
     ngtcp2_unreachable();
   }
 }
 
 const uint8_t *ngtcp2_get_uvarint(uint64_t *dest, const uint8_t *p) {
-  size_t len;
-
-  *dest = get_uvarint(&len, p);
-
-  return p + len;
+  return get_uvarint(dest, p);
 }
 
 const uint8_t *ngtcp2_get_varint(int64_t *dest, const uint8_t *p) {
-  size_t len;
-
-  *dest = (int64_t)get_uvarint(&len, p);
-
-  return p + len;
+  return get_uvarint((uint64_t *)dest, p);
 }
 
 int64_t ngtcp2_get_pkt_num(const uint8_t *p, size_t pkt_numlen) {
@@ -126,13 +113,13 @@ int64_t ngtcp2_get_pkt_num(const uint8_t *p, size_t pkt_numlen) {
   case 1:
     return *p;
   case 2:
-    ngtcp2_get_uint16(&s, p);
+    ngtcp2_get_uint16be(&s, p);
     return (int64_t)s;
   case 3:
-    ngtcp2_get_uint24(&l, p);
+    ngtcp2_get_uint24be(&l, p);
     return (int64_t)l;
   case 4:
-    ngtcp2_get_uint32(&l, p);
+    ngtcp2_get_uint32be(&l, p);
     return (int64_t)l;
   default:
     ngtcp2_unreachable();
@@ -144,11 +131,6 @@ uint8_t *ngtcp2_put_uint64be(uint8_t *p, uint64_t n) {
   return ngtcp2_cpymem(p, (const uint8_t *)&n, sizeof(n));
 }
 
-uint8_t *ngtcp2_put_uint48be(uint8_t *p, uint64_t n) {
-  n = ngtcp2_htonl64(n);
-  return ngtcp2_cpymem(p, ((const uint8_t *)&n) + 2, 6);
-}
-
 uint8_t *ngtcp2_put_uint32be(uint8_t *p, uint32_t n) {
   n = ngtcp2_htonl(n);
   return ngtcp2_cpymem(p, (const uint8_t *)&n, sizeof(n));
@@ -207,14 +189,11 @@ uint8_t *ngtcp2_put_pkt_num(uint8_t *p, int64_t pkt_num, size_t len) {
     *p++ = (uint8_t)pkt_num;
     return p;
   case 2:
-    ngtcp2_put_uint16be(p, (uint16_t)pkt_num);
-    return p + 2;
+    return ngtcp2_put_uint16be(p, (uint16_t)pkt_num);
   case 3:
-    ngtcp2_put_uint24be(p, (uint32_t)pkt_num);
-    return p + 3;
+    return ngtcp2_put_uint24be(p, (uint32_t)pkt_num);
   case 4:
-    ngtcp2_put_uint32be(p, (uint32_t)pkt_num);
-    return p + 4;
+    return ngtcp2_put_uint32be(p, (uint32_t)pkt_num);
   default:
     ngtcp2_unreachable();
   }
@@ -238,54 +217,6 @@ size_t ngtcp2_put_uvarintlen(uint64_t n) {
   return 8;
 }
 
-int64_t ngtcp2_nth_server_bidi_id(uint64_t n) {
-  if (n == 0) {
-    return 0;
-  }
-
-  if ((NGTCP2_MAX_VARINT >> 2) < n - 1) {
-    return NGTCP2_MAX_SERVER_STREAM_ID_BIDI;
-  }
-
-  return (int64_t)(((n - 1) << 2) | 0x01);
-}
-
-int64_t ngtcp2_nth_client_bidi_id(uint64_t n) {
-  if (n == 0) {
-    return 0;
-  }
-
-  if ((NGTCP2_MAX_VARINT >> 2) < n - 1) {
-    return NGTCP2_MAX_CLIENT_STREAM_ID_BIDI;
-  }
-
-  return (int64_t)((n - 1) << 2);
-}
-
-int64_t ngtcp2_nth_server_uni_id(uint64_t n) {
-  if (n == 0) {
-    return 0;
-  }
-
-  if ((NGTCP2_MAX_VARINT >> 2) < n - 1) {
-    return NGTCP2_MAX_SERVER_STREAM_ID_UNI;
-  }
-
-  return (int64_t)(((n - 1) << 2) | 0x03);
-}
-
-int64_t ngtcp2_nth_client_uni_id(uint64_t n) {
-  if (n == 0) {
-    return 0;
-  }
-
-  if ((NGTCP2_MAX_VARINT >> 2) < n - 1) {
-    return NGTCP2_MAX_CLIENT_STREAM_ID_UNI;
-  }
-
-  return (int64_t)(((n - 1) << 2) | 0x02);
-}
-
 uint64_t ngtcp2_ord_stream_id(int64_t stream_id) {
   return (uint64_t)(stream_id >> 2) + 1;
 }
