@@ -27,7 +27,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif /* defined(HAVE_CONFIG_H) */
 
 #include <ngtcp2/ngtcp2.h>
 
@@ -41,7 +41,6 @@
 #define NGTCP2_LONG_RESERVED_BIT_MASK 0x0c
 
 /* Short header specific macros */
-#define NGTCP2_SHORT_SPIN_BIT_MASK 0x20
 #define NGTCP2_SHORT_RESERVED_BIT_MASK 0x18
 #define NGTCP2_SHORT_KEY_PHASE_BIT 0x04
 
@@ -53,10 +52,15 @@
    LENGTH<1> + PKN<1> */
 #define NGTCP2_MIN_LONG_HEADERLEN (1 + 4 + 1 + 1 + 1 + 1)
 
+/* STREAM frame specific macros */
 #define NGTCP2_STREAM_FIN_BIT 0x01
 #define NGTCP2_STREAM_LEN_BIT 0x02
 #define NGTCP2_STREAM_OFF_BIT 0x04
 
+/* NGTCP2_MIN_QUIC_PKTLEN is the minimum length of a valid QUIC
+   packet. */
+#define NGTCP2_MIN_QUIC_PKTLEN 21
+
 /* NGTCP2_STREAM_OVERHEAD is the maximum number of bytes required
    other than payload for STREAM frame.  That is from type field to
    the beginning of the payload. */
@@ -72,9 +76,6 @@
    the beginning of the payload. */
 #define NGTCP2_DATAGRAM_OVERHEAD (1 + 8)
 
-/* NGTCP2_MIN_FRAME_PAYLOADLEN is the minimum frame payload length. */
-#define NGTCP2_MIN_FRAME_PAYLOADLEN 16
-
 /* NGTCP2_MAX_SERVER_STREAM_ID_BIDI is the maximum bidirectional
    server stream ID. */
 #define NGTCP2_MAX_SERVER_STREAM_ID_BIDI ((int64_t)0x3ffffffffffffffdll)
@@ -95,16 +96,15 @@
 /* NGTCP2_MAX_PKT_NUM is the maximum packet number. */
 #define NGTCP2_MAX_PKT_NUM ((int64_t)((1ll << 62) - 1))
 
-/* NGTCP2_MIN_PKT_EXPANDLEN is the minimum packet size expansion in
-   addition to the minimum DCID length to hide/trigger Stateless
-   Reset. */
+/* NGTCP2_MIN_PKT_EXPANDLEN is the minimum packet size expansion to
+   hide/trigger Stateless Reset. */
 #define NGTCP2_MIN_PKT_EXPANDLEN 22
 
 /* NGTCP2_RETRY_TAGLEN is the length of Retry packet integrity tag. */
 #define NGTCP2_RETRY_TAGLEN 16
 
-/* NGTCP2_HARD_MAX_UDP_PAYLOAD_SIZE is the maximum UDP payload size
-   that this library can write. */
+/* NGTCP2_HARD_MAX_UDP_PAYLOAD_SIZE is the maximum UDP datagram
+   payload size that this library can write. */
 #define NGTCP2_HARD_MAX_UDP_PAYLOAD_SIZE ((1 << 24) - 1)
 
 /* NGTCP2_PKT_LENGTHLEN is the number of bytes that is occupied by
@@ -124,62 +124,80 @@
    v1. */
 #define NGTCP2_PKT_TYPE_RETRY_V1 0x3
 
-/* NGTCP2_PKT_TYPE_INITIAL_V2_DRAFT is Initial long header packet type
-   for QUIC v2 draft. */
-#define NGTCP2_PKT_TYPE_INITIAL_V2_DRAFT 0x1
-/* NGTCP2_PKT_TYPE_0RTT_V2_DRAFT is 0RTT long header packet type for
-   QUIC v2 draft. */
-#define NGTCP2_PKT_TYPE_0RTT_V2_DRAFT 0x2
-/* NGTCP2_PKT_TYPE_HANDSHAKE_V2_DRAFT is Handshake long header packet
-   type for QUIC v2 draft. */
-#define NGTCP2_PKT_TYPE_HANDSHAKE_V2_DRAFT 0x3
-/* NGTCP2_PKT_TYPE_RETRY_V2_DRAFT is Retry long header packet type for
-   QUIC v2 draft. */
-#define NGTCP2_PKT_TYPE_RETRY_V2_DRAFT 0x0
+/* NGTCP2_PKT_TYPE_INITIAL_V2 is Initial long header packet type for
+   QUIC v2. */
+#define NGTCP2_PKT_TYPE_INITIAL_V2 0x1
+/* NGTCP2_PKT_TYPE_0RTT_V2 is 0RTT long header packet type for QUIC
+   v2. */
+#define NGTCP2_PKT_TYPE_0RTT_V2 0x2
+/* NGTCP2_PKT_TYPE_HANDSHAKE_V2 is Handshake long header packet type
+   for QUIC v2. */
+#define NGTCP2_PKT_TYPE_HANDSHAKE_V2 0x3
+/* NGTCP2_PKT_TYPE_RETRY_V2 is Retry long header packet type for QUIC
+   v2. */
+#define NGTCP2_PKT_TYPE_RETRY_V2 0x0
+
+/* NGTCP2_MIN_STREAM_DATALEN is the minimum length of STREAM frame to
+   avoid too small frame.  It is not always enforced for various
+   reasons.  For example, due to flow control, we might have fewer
+   bytes available to send.  Therefore, it is only applied when the
+   length of data to send is larger than this limit. */
+#define NGTCP2_MIN_STREAM_DATALEN 256
+
+/* NGTCP2_MAX_STREAM_DATACNT is the maximum number of ngtcp2_vec that
+   a ngtcp2_stream can include. */
+#define NGTCP2_MAX_STREAM_DATACNT 256
+
+typedef struct ngtcp2_pcg32 ngtcp2_pcg32;
 
 typedef struct ngtcp2_pkt_retry {
   ngtcp2_cid odcid;
-  ngtcp2_vec token;
+  uint8_t *token;
+  size_t tokenlen;
   uint8_t tag[NGTCP2_RETRY_TAGLEN];
 } ngtcp2_pkt_retry;
 
-typedef enum {
-  NGTCP2_FRAME_PADDING = 0x00,
-  NGTCP2_FRAME_PING = 0x01,
-  NGTCP2_FRAME_ACK = 0x02,
-  NGTCP2_FRAME_ACK_ECN = 0x03,
-  NGTCP2_FRAME_RESET_STREAM = 0x04,
-  NGTCP2_FRAME_STOP_SENDING = 0x05,
-  NGTCP2_FRAME_CRYPTO = 0x06,
-  NGTCP2_FRAME_NEW_TOKEN = 0x07,
-  NGTCP2_FRAME_STREAM = 0x08,
-  NGTCP2_FRAME_MAX_DATA = 0x10,
-  NGTCP2_FRAME_MAX_STREAM_DATA = 0x11,
-  NGTCP2_FRAME_MAX_STREAMS_BIDI = 0x12,
-  NGTCP2_FRAME_MAX_STREAMS_UNI = 0x13,
-  NGTCP2_FRAME_DATA_BLOCKED = 0x14,
-  NGTCP2_FRAME_STREAM_DATA_BLOCKED = 0x15,
-  NGTCP2_FRAME_STREAMS_BLOCKED_BIDI = 0x16,
-  NGTCP2_FRAME_STREAMS_BLOCKED_UNI = 0x17,
-  NGTCP2_FRAME_NEW_CONNECTION_ID = 0x18,
-  NGTCP2_FRAME_RETIRE_CONNECTION_ID = 0x19,
-  NGTCP2_FRAME_PATH_CHALLENGE = 0x1a,
-  NGTCP2_FRAME_PATH_RESPONSE = 0x1b,
-  NGTCP2_FRAME_CONNECTION_CLOSE = 0x1c,
-  NGTCP2_FRAME_CONNECTION_CLOSE_APP = 0x1d,
-  NGTCP2_FRAME_HANDSHAKE_DONE = 0x1e,
-  NGTCP2_FRAME_DATAGRAM = 0x30,
-  NGTCP2_FRAME_DATAGRAM_LEN = 0x31,
-} ngtcp2_frame_type;
+#define NGTCP2_FRAME_PADDING 0x00
+#define NGTCP2_FRAME_PING 0x01
+#define NGTCP2_FRAME_ACK 0x02
+#define NGTCP2_FRAME_ACK_ECN 0x03
+#define NGTCP2_FRAME_RESET_STREAM 0x04
+#define NGTCP2_FRAME_STOP_SENDING 0x05
+#define NGTCP2_FRAME_CRYPTO 0x06
+#define NGTCP2_FRAME_NEW_TOKEN 0x07
+#define NGTCP2_FRAME_STREAM 0x08
+#define NGTCP2_FRAME_MAX_DATA 0x10
+#define NGTCP2_FRAME_MAX_STREAM_DATA 0x11
+#define NGTCP2_FRAME_MAX_STREAMS_BIDI 0x12
+#define NGTCP2_FRAME_MAX_STREAMS_UNI 0x13
+#define NGTCP2_FRAME_DATA_BLOCKED 0x14
+#define NGTCP2_FRAME_STREAM_DATA_BLOCKED 0x15
+#define NGTCP2_FRAME_STREAMS_BLOCKED_BIDI 0x16
+#define NGTCP2_FRAME_STREAMS_BLOCKED_UNI 0x17
+#define NGTCP2_FRAME_NEW_CONNECTION_ID 0x18
+#define NGTCP2_FRAME_RETIRE_CONNECTION_ID 0x19
+#define NGTCP2_FRAME_PATH_CHALLENGE 0x1a
+#define NGTCP2_FRAME_PATH_RESPONSE 0x1b
+#define NGTCP2_FRAME_CONNECTION_CLOSE 0x1c
+#define NGTCP2_FRAME_CONNECTION_CLOSE_APP 0x1d
+#define NGTCP2_FRAME_HANDSHAKE_DONE 0x1e
+#define NGTCP2_FRAME_DATAGRAM 0x30
+#define NGTCP2_FRAME_DATAGRAM_LEN 0x31
 
+/* ngtcp2_stream represents STREAM and CRYPTO frames. */
 typedef struct ngtcp2_stream {
-  uint8_t type;
+  uint64_t type;
   /**
    * flags of decoded STREAM frame.  This gets ignored when encoding
-   * STREAM frame.
+   * STREAM frame.  CRYPTO frame does not include this field, and must
+   * set it to 0.
    */
   uint8_t flags;
+  /* CRYPTO frame does not include this field, and must set it to
+     0. */
   uint8_t fin;
+  /* CRYPTO frame does not include this field, and must set it to
+     0. */
   int64_t stream_id;
   uint64_t offset;
   /* datacnt is the number of elements that data contains.  Although
@@ -196,7 +214,7 @@ typedef struct ngtcp2_ack_range {
 } ngtcp2_ack_range;
 
 typedef struct ngtcp2_ack {
-  uint8_t type;
+  uint64_t type;
   int64_t largest_ack;
   uint64_t ack_delay;
   /**
@@ -215,7 +233,7 @@ typedef struct ngtcp2_ack {
 } ngtcp2_ack;
 
 typedef struct ngtcp2_padding {
-  uint8_t type;
+  uint64_t type;
   /**
    * The length of contiguous PADDING frames.
    */
@@ -223,14 +241,14 @@ typedef struct ngtcp2_padding {
 } ngtcp2_padding;
 
 typedef struct ngtcp2_reset_stream {
-  uint8_t type;
+  uint64_t type;
   int64_t stream_id;
   uint64_t app_error_code;
   uint64_t final_size;
 } ngtcp2_reset_stream;
 
 typedef struct ngtcp2_connection_close {
-  uint8_t type;
+  uint64_t type;
   uint64_t error_code;
   uint64_t frame_type;
   size_t reasonlen;
@@ -238,7 +256,7 @@ typedef struct ngtcp2_connection_close {
 } ngtcp2_connection_close;
 
 typedef struct ngtcp2_max_data {
-  uint8_t type;
+  uint64_t type;
   /**
    * max_data is Maximum Data.
    */
@@ -246,38 +264,38 @@ typedef struct ngtcp2_max_data {
 } ngtcp2_max_data;
 
 typedef struct ngtcp2_max_stream_data {
-  uint8_t type;
+  uint64_t type;
   int64_t stream_id;
   uint64_t max_stream_data;
 } ngtcp2_max_stream_data;
 
 typedef struct ngtcp2_max_streams {
-  uint8_t type;
+  uint64_t type;
   uint64_t max_streams;
 } ngtcp2_max_streams;
 
 typedef struct ngtcp2_ping {
-  uint8_t type;
+  uint64_t type;
 } ngtcp2_ping;
 
 typedef struct ngtcp2_data_blocked {
-  uint8_t type;
+  uint64_t type;
   uint64_t offset;
 } ngtcp2_data_blocked;
 
 typedef struct ngtcp2_stream_data_blocked {
-  uint8_t type;
+  uint64_t type;
   int64_t stream_id;
   uint64_t offset;
 } ngtcp2_stream_data_blocked;
 
 typedef struct ngtcp2_streams_blocked {
-  uint8_t type;
+  uint64_t type;
   uint64_t max_streams;
 } ngtcp2_streams_blocked;
 
 typedef struct ngtcp2_new_connection_id {
-  uint8_t type;
+  uint64_t type;
   uint64_t seq;
   uint64_t retire_prior_to;
   ngtcp2_cid cid;
@@ -285,48 +303,38 @@ typedef struct ngtcp2_new_connection_id {
 } ngtcp2_new_connection_id;
 
 typedef struct ngtcp2_stop_sending {
-  uint8_t type;
+  uint64_t type;
   int64_t stream_id;
   uint64_t app_error_code;
 } ngtcp2_stop_sending;
 
 typedef struct ngtcp2_path_challenge {
-  uint8_t type;
+  uint64_t type;
   uint8_t data[NGTCP2_PATH_CHALLENGE_DATALEN];
 } ngtcp2_path_challenge;
 
 typedef struct ngtcp2_path_response {
-  uint8_t type;
+  uint64_t type;
   uint8_t data[NGTCP2_PATH_CHALLENGE_DATALEN];
 } ngtcp2_path_response;
 
-typedef struct ngtcp2_crypto {
-  uint8_t type;
-  uint64_t offset;
-  /* datacnt is the number of elements that data contains.  Although
-     the length of data is 1 in this definition, the library may
-     allocate extra bytes to hold more elements. */
-  size_t datacnt;
-  /* data is the array of ngtcp2_vec which references data. */
-  ngtcp2_vec data[1];
-} ngtcp2_crypto;
-
 typedef struct ngtcp2_new_token {
-  uint8_t type;
-  ngtcp2_vec token;
+  uint64_t type;
+  uint8_t *token;
+  size_t tokenlen;
 } ngtcp2_new_token;
 
 typedef struct ngtcp2_retire_connection_id {
-  uint8_t type;
+  uint64_t type;
   uint64_t seq;
 } ngtcp2_retire_connection_id;
 
 typedef struct ngtcp2_handshake_done {
-  uint8_t type;
+  uint64_t type;
 } ngtcp2_handshake_done;
 
 typedef struct ngtcp2_datagram {
-  uint8_t type;
+  uint64_t type;
   /* dgram_id is an opaque identifier chosen by an application. */
   uint64_t dgram_id;
   /* datacnt is the number of elements that data contains. */
@@ -341,7 +349,7 @@ typedef struct ngtcp2_datagram {
 } ngtcp2_datagram;
 
 typedef union ngtcp2_frame {
-  uint8_t type;
+  uint64_t type;
   ngtcp2_stream stream;
   ngtcp2_ack ack;
   ngtcp2_padding padding;
@@ -358,11 +366,15 @@ typedef union ngtcp2_frame {
   ngtcp2_stop_sending stop_sending;
   ngtcp2_path_challenge path_challenge;
   ngtcp2_path_response path_response;
-  ngtcp2_crypto crypto;
   ngtcp2_new_token new_token;
   ngtcp2_retire_connection_id retire_connection_id;
   ngtcp2_handshake_done handshake_done;
   ngtcp2_datagram datagram;
+  /* Extend ngtcp2_frame so that ngtcp2_stream has at least additional
+     3 ngtcp2_vec, totaling 4 slots, which can store HEADERS header,
+     HEADERS payload, DATA header, and DATA payload in the standard
+     sized ngtcp2_frame_chain. */
+  uint8_t pad[sizeof(ngtcp2_stream) + sizeof(ngtcp2_vec) * 3];
 } ngtcp2_frame;
 
 typedef struct ngtcp2_pkt_chain ngtcp2_pkt_chain;
@@ -377,7 +389,7 @@ struct ngtcp2_pkt_chain {
   uint8_t *pkt;
   /* pktlen is length of a QUIC packet. */
   size_t pktlen;
-  /* dgramlen is length of UDP datagram that a QUIC packet is
+  /* dgramlen is length of UDP datagram payload that a QUIC packet is
      included. */
   size_t dgramlen;
   ngtcp2_tstamp ts;
@@ -409,22 +421,20 @@ void ngtcp2_pkt_chain_del(ngtcp2_pkt_chain *pc, const ngtcp2_mem *mem);
 
 /*
  * ngtcp2_pkt_hd_init initializes |hd| with the given values.  If
- * |dcid| and/or |scid| is NULL, DCID and SCID of |hd| is empty
- * respectively.  |pkt_numlen| is the number of bytes used to encode
- * |pkt_num| and either 1, 2, or 4.  |version| is QUIC version for
- * long header.  |len| is the length field of Initial, 0RTT, and
- * Handshake packets.
+ * |dcid| and/or |scid| is NULL, Destination Connection ID and/or
+ * Source Connection ID of |hd| is empty respectively.  |pkt_numlen|
+ * is the number of bytes used to encode |pkt_num| and either 1, 2, or
+ * 4.  |version| is QUIC version for long header.
  */
 void ngtcp2_pkt_hd_init(ngtcp2_pkt_hd *hd, uint8_t flags, uint8_t type,
                         const ngtcp2_cid *dcid, const ngtcp2_cid *scid,
-                        int64_t pkt_num, size_t pkt_numlen, uint32_t version,
-                        size_t len);
+                        int64_t pkt_num, size_t pkt_numlen, uint32_t version);
 
 /*
  * ngtcp2_pkt_encode_hd_long encodes |hd| as QUIC long header into
  * |out| which has length |outlen|.  It returns the number of bytes
- * written into |outlen| if it succeeds, or one of the following
- * negative error codes:
+ * written into |out| if it succeeds, or one of the following negative
+ * error codes:
  *
  * NGTCP2_ERR_NOBUF
  *     Buffer is too short
@@ -435,8 +445,8 @@ ngtcp2_ssize ngtcp2_pkt_encode_hd_long(uint8_t *out, size_t outlen,
 /*
  * ngtcp2_pkt_encode_hd_short encodes |hd| as QUIC short header into
  * |out| which has length |outlen|.  It returns the number of bytes
- * written into |outlen| if it succeeds, or one of the following
- * negative error codes:
+ * written into |out| if it succeeds, or one of the following negative
+ * error codes:
  *
  * NGTCP2_ERR_NOBUF
  *     Buffer is too short
@@ -463,7 +473,7 @@ ngtcp2_ssize ngtcp2_pkt_decode_frame(ngtcp2_frame *dest, const uint8_t *payload,
 /**
  * @function
  *
- * `ngtcp2_pkt_encode_frame` encodes a frame |fm| into the buffer
+ * `ngtcp2_pkt_encode_frame` encodes a frame |fr| into the buffer
  * pointed by |out| of length |outlen|.
  *
  * This function returns the number of bytes written to the buffer, or
@@ -549,7 +559,7 @@ ngtcp2_ssize ngtcp2_pkt_decode_ack_frame(ngtcp2_ack *dest,
 /*
  * ngtcp2_pkt_decode_padding_frame decodes contiguous PADDING frames
  * from |payload| of length |payloadlen|.  It continues to parse
- * frames as long as the frame type is PADDING.  This finishes when it
+ * frames as long as the frame type is PADDING.  It finishes when it
  * encounters the frame type which is not PADDING, or all input data
  * is read.  The first byte (payload[0]) must be NGTCP2_FRAME_PADDING.
  * This function returns the exact number of bytes read to decode
@@ -579,7 +589,7 @@ ngtcp2_ssize ngtcp2_pkt_decode_reset_stream_frame(ngtcp2_reset_stream *dest,
  * ngtcp2_pkt_decode_connection_close_frame decodes CONNECTION_CLOSE
  * frame from |payload| of length |payloadlen|.  The result is stored
  * in the object pointed by |dest|.  CONNECTION_CLOSE frame must start
- * at payload[0].  This function finishes it decodes one
+ * at payload[0].  This function finishes when it decodes one
  * CONNECTION_CLOSE frame, and returns the exact number of bytes read
  * to decode a frame if it succeeds, or one of the following negative
  * error codes:
@@ -588,7 +598,7 @@ ngtcp2_ssize ngtcp2_pkt_decode_reset_stream_frame(ngtcp2_reset_stream *dest,
  *     Payload is too short to include CONNECTION_CLOSE frame.
  */
 ngtcp2_ssize ngtcp2_pkt_decode_connection_close_frame(
-    ngtcp2_connection_close *dest, const uint8_t *payload, size_t payloadlen);
+  ngtcp2_connection_close *dest, const uint8_t *payload, size_t payloadlen);
 
 /*
  * ngtcp2_pkt_decode_max_data_frame decodes MAX_DATA frame from
@@ -618,7 +628,7 @@ ngtcp2_ssize ngtcp2_pkt_decode_max_data_frame(ngtcp2_max_data *dest,
  *     Payload is too short to include MAX_STREAM_DATA frame.
  */
 ngtcp2_ssize ngtcp2_pkt_decode_max_stream_data_frame(
-    ngtcp2_max_stream_data *dest, const uint8_t *payload, size_t payloadlen);
+  ngtcp2_max_stream_data *dest, const uint8_t *payload, size_t payloadlen);
 
 /*
  * ngtcp2_pkt_decode_max_streams_frame decodes MAX_STREAMS frame from
@@ -674,10 +684,8 @@ ngtcp2_ssize ngtcp2_pkt_decode_data_blocked_frame(ngtcp2_data_blocked *dest,
  * NGTCP2_ERR_FRAME_ENCODING
  *     Payload is too short to include STREAM_DATA_BLOCKED frame.
  */
-ngtcp2_ssize
-ngtcp2_pkt_decode_stream_data_blocked_frame(ngtcp2_stream_data_blocked *dest,
-                                            const uint8_t *payload,
-                                            size_t payloadlen);
+ngtcp2_ssize ngtcp2_pkt_decode_stream_data_blocked_frame(
+  ngtcp2_stream_data_blocked *dest, const uint8_t *payload, size_t payloadlen);
 
 /*
  * ngtcp2_pkt_decode_streams_blocked_frame decodes STREAMS_BLOCKED
@@ -692,7 +700,7 @@ ngtcp2_pkt_decode_stream_data_blocked_frame(ngtcp2_stream_data_blocked *dest,
  *     Payload is too short to include STREAMS_BLOCKED frame.
  */
 ngtcp2_ssize ngtcp2_pkt_decode_streams_blocked_frame(
-    ngtcp2_streams_blocked *dest, const uint8_t *payload, size_t payloadlen);
+  ngtcp2_streams_blocked *dest, const uint8_t *payload, size_t payloadlen);
 
 /*
  * ngtcp2_pkt_decode_new_connection_id_frame decodes NEW_CONNECTION_ID
@@ -705,11 +713,11 @@ ngtcp2_ssize ngtcp2_pkt_decode_streams_blocked_frame(
  *
  * NGTCP2_ERR_FRAME_ENCODING
  *     Payload is too short to include NEW_CONNECTION_ID frame; or the
- *     length of CID is strictly less than NGTCP2_MIN_CIDLEN or
- *     greater than NGTCP2_MAX_CIDLEN.
+ *     length of Connection ID is strictly less than NGTCP2_MIN_CIDLEN
+ *     or greater than NGTCP2_MAX_CIDLEN.
  */
 ngtcp2_ssize ngtcp2_pkt_decode_new_connection_id_frame(
-    ngtcp2_new_connection_id *dest, const uint8_t *payload, size_t payloadlen);
+  ngtcp2_new_connection_id *dest, const uint8_t *payload, size_t payloadlen);
 
 /*
  * ngtcp2_pkt_decode_stop_sending_frame decodes STOP_SENDING frame
@@ -770,7 +778,7 @@ ngtcp2_ssize ngtcp2_pkt_decode_path_response_frame(ngtcp2_path_response *dest,
  * NGTCP2_ERR_FRAME_ENCODING
  *     Payload is too short to include CRYPTO frame.
  */
-ngtcp2_ssize ngtcp2_pkt_decode_crypto_frame(ngtcp2_crypto *dest,
+ngtcp2_ssize ngtcp2_pkt_decode_crypto_frame(ngtcp2_stream *dest,
                                             const uint8_t *payload,
                                             size_t payloadlen);
 
@@ -790,20 +798,19 @@ ngtcp2_ssize ngtcp2_pkt_decode_new_token_frame(ngtcp2_new_token *dest,
                                                size_t payloadlen);
 
 /*
- * ngtcp2_pkt_decode_retire_connection_id_frame decodes RETIRE_CONNECTION_ID
- * frame from |payload| of length |payloadlen|.  The result is stored in the
- * object pointed by |dest|.  RETIRE_CONNECTION_ID frame must start at
- * payload[0].  This function finishes when it decodes one RETIRE_CONNECTION_ID
- * frame, and returns the exact number of bytes read to decode a frame
- * if it succeeds, or one of the following negative error codes:
+ * ngtcp2_pkt_decode_retire_connection_id_frame decodes
+ * RETIRE_CONNECTION_ID frame from |payload| of length |payloadlen|.
+ * The result is stored in the object pointed by |dest|.
+ * RETIRE_CONNECTION_ID frame must start at payload[0].  This function
+ * finishes when it decodes one RETIRE_CONNECTION_ID frame, and
+ * returns the exact number of bytes read to decode a frame if it
+ * succeeds, or one of the following negative error codes:
  *
  * NGTCP2_ERR_FRAME_ENCODING
  *     Payload is too short to include RETIRE_CONNECTION_ID frame.
  */
-ngtcp2_ssize
-ngtcp2_pkt_decode_retire_connection_id_frame(ngtcp2_retire_connection_id *dest,
-                                             const uint8_t *payload,
-                                             size_t payloadlen);
+ngtcp2_ssize ngtcp2_pkt_decode_retire_connection_id_frame(
+  ngtcp2_retire_connection_id *dest, const uint8_t *payload, size_t payloadlen);
 
 /*
  * ngtcp2_pkt_decode_handshake_done_frame decodes HANDSHAKE_DONE frame
@@ -852,9 +859,6 @@ ngtcp2_ssize ngtcp2_pkt_encode_stream_frame(uint8_t *out, size_t outlen,
  * ngtcp2_pkt_encode_ack_frame encodes ACK frame |fr| into the buffer
  * pointed by |out| of length |outlen|.
  *
- * This function assigns <the serialized frame type> &
- * ~NGTCP2_FRAME_ACK to fr->flags.
- *
  * This function returns the number of bytes written if it succeeds,
  * or one of the following negative error codes:
  *
@@ -862,7 +866,7 @@ ngtcp2_ssize ngtcp2_pkt_encode_stream_frame(uint8_t *out, size_t outlen,
  *     Buffer does not have enough capacity to write a frame.
  */
 ngtcp2_ssize ngtcp2_pkt_encode_ack_frame(uint8_t *out, size_t outlen,
-                                         ngtcp2_ack *fr);
+                                         const ngtcp2_ack *fr);
 
 /*
  * ngtcp2_pkt_encode_padding_frame encodes PADDING frame |fr| into the
@@ -986,7 +990,7 @@ ngtcp2_pkt_encode_data_blocked_frame(uint8_t *out, size_t outlen,
  *     Buffer does not have enough capacity to write a frame.
  */
 ngtcp2_ssize ngtcp2_pkt_encode_stream_data_blocked_frame(
-    uint8_t *out, size_t outlen, const ngtcp2_stream_data_blocked *fr);
+  uint8_t *out, size_t outlen, const ngtcp2_stream_data_blocked *fr);
 
 /*
  * ngtcp2_pkt_encode_streams_blocked_frame encodes STREAMS_BLOCKED
@@ -1069,7 +1073,7 @@ ngtcp2_pkt_encode_path_response_frame(uint8_t *out, size_t outlen,
  *     Buffer does not have enough capacity to write a frame.
  */
 ngtcp2_ssize ngtcp2_pkt_encode_crypto_frame(uint8_t *out, size_t outlen,
-                                            const ngtcp2_crypto *fr);
+                                            const ngtcp2_stream *fr);
 
 /*
  * ngtcp2_pkt_encode_new_token_frame encodes NEW_TOKEN frame |fr| into
@@ -1085,8 +1089,9 @@ ngtcp2_ssize ngtcp2_pkt_encode_new_token_frame(uint8_t *out, size_t outlen,
                                                const ngtcp2_new_token *fr);
 
 /*
- * ngtcp2_pkt_encode_retire_connection_id_frame encodes RETIRE_CONNECTION_ID
- * frame |fr| into the buffer pointed by |out| of length |outlen|.
+ * ngtcp2_pkt_encode_retire_connection_id_frame encodes
+ * RETIRE_CONNECTION_ID frame |fr| into the buffer pointed by |out| of
+ * length |outlen|.
  *
  * This function returns the number of bytes written if it succeeds,
  * or one of the following negative error codes:
@@ -1095,7 +1100,7 @@ ngtcp2_ssize ngtcp2_pkt_encode_new_token_frame(uint8_t *out, size_t outlen,
  *     Buffer does not have enough capacity to write a frame.
  */
 ngtcp2_ssize ngtcp2_pkt_encode_retire_connection_id_frame(
-    uint8_t *out, size_t outlen, const ngtcp2_retire_connection_id *fr);
+  uint8_t *out, size_t outlen, const ngtcp2_retire_connection_id *fr);
 
 /*
  * ngtcp2_pkt_encode_handshake_done_frame encodes HANDSHAKE_DONE frame
@@ -1125,30 +1130,35 @@ ngtcp2_ssize ngtcp2_pkt_encode_datagram_frame(uint8_t *out, size_t outlen,
                                               const ngtcp2_datagram *fr);
 
 /*
- * ngtcp2_pkt_adjust_pkt_num find the full 64 bits packet number for
- * |pkt_num|, which is expected to be least significant |n| bits.  The
+ * ngtcp2_pkt_adjust_pkt_num finds the full 62 bits packet number for
+ * |pkt_num|, which is encoded in |pkt_numlen| bytes.  The
  * |max_pkt_num| is the highest successfully authenticated packet
  * number.
  */
 int64_t ngtcp2_pkt_adjust_pkt_num(int64_t max_pkt_num, int64_t pkt_num,
-                                  size_t n);
+                                  size_t pkt_numlen);
 
 /*
- * ngtcp2_pkt_validate_ack checks that ack is malformed or not.
+ * ngtcp2_pkt_validate_ack verifies whether |fr| is malformed or not.
+ * |min_pkt_num| is the minimum packet number that a local endpoint
+ * sends.  It is an error to receive acknowledgements for a packet
+ * less than |min_pkt_num|.
  *
  * This function returns 0 if it succeeds, or one of the following
  * negative error codes:
  *
  * NGTCP2_ERR_ACK_FRAME
  *     ACK frame is malformed
+ * NGTCP2_ERR_PROTO
+ *     |fr| contains a packet number less than |min_pkt_num|.
  */
-int ngtcp2_pkt_validate_ack(ngtcp2_ack *fr);
+int ngtcp2_pkt_validate_ack(const ngtcp2_ack *fr, int64_t min_pkt_num);
 
 /*
  * ngtcp2_pkt_stream_max_datalen returns the maximum number of bytes
  * which can be sent for stream denoted by |stream_id|.  |offset| is
- * an offset of within the stream.  |len| is the estimated number of
- * bytes to be sent.  |left| is the size of buffer.  If |left| is too
+ * an offset within the stream.  |len| is the estimated number of
+ * bytes to send.  |left| is the size of buffer.  If |left| is too
  * small to write STREAM frame, this function returns (size_t)-1.
  */
 size_t ngtcp2_pkt_stream_max_datalen(int64_t stream_id, uint64_t offset,
@@ -1156,10 +1166,10 @@ size_t ngtcp2_pkt_stream_max_datalen(int64_t stream_id, uint64_t offset,
 
 /*
  * ngtcp2_pkt_crypto_max_datalen returns the maximum number of bytes
- * which can be sent for crypto stream.  |offset| is an offset of
- * within the crypto stream.  |len| is the estimated number of bytes
- * to be sent.  |left| is the size of buffer.  If |left| is too small
- * to write CRYPTO frame, this function returns (size_t)-1.
+ * which can be sent for crypto stream.  |offset| is an offset within
+ * the crypto stream.  |len| is the estimated number of bytes to send.
+ * |left| is the size of buffer.  If |left| is too small to write
+ * CRYPTO frame, this function returns (size_t)-1.
  */
 size_t ngtcp2_pkt_crypto_max_datalen(uint64_t offset, size_t len, size_t left);
 
@@ -1192,8 +1202,8 @@ int ngtcp2_pkt_verify_reserved_bits(uint8_t c);
  *     Buffer is too short.
  */
 ngtcp2_ssize ngtcp2_pkt_encode_pseudo_retry(
-    uint8_t *dest, size_t destlen, const ngtcp2_pkt_hd *hd, uint8_t unused,
-    const ngtcp2_cid *odcid, const uint8_t *token, size_t tokenlen);
+  uint8_t *dest, size_t destlen, const ngtcp2_pkt_hd *hd, uint8_t unused,
+  const ngtcp2_cid *odcid, const uint8_t *token, size_t tokenlen);
 
 /*
  * ngtcp2_pkt_verify_retry_tag verifies Retry packet.  The buffer
@@ -1230,4 +1240,81 @@ uint8_t ngtcp2_pkt_versioned_type(uint32_t version, uint32_t pkt_type);
  */
 uint8_t ngtcp2_pkt_get_type_long(uint32_t version, uint8_t c);
 
-#endif /* NGTCP2_PKT_H */
+/*
+ * ngtcp2_pkt_split_vec_rand appends ngtcp2_vec at most |max_add|
+ * times to the array pointed by |data| of length |datacnt| by
+ * splitting the existing ngtcp2_vec into two.  Which ngtcp2_vec to
+ * split is chosen randomly.  |offsets| contains the offset of each
+ * ngtcp2_vec pointed by |data|.  |offsets| is also updated.  The
+ * arrays must have the capacity at least |datacnt| + |max_add|.
+ * |pcg| is a random number generator.
+ *
+ * This function returns |datacnt| plus the number of ngtcp2_vec that
+ * are appended.
+ */
+size_t ngtcp2_pkt_split_vec_rand(ngtcp2_vec *data, size_t datacnt,
+                                 uint64_t *offsets, ngtcp2_pcg32 *pcg,
+                                 size_t max_add);
+
+/*
+ * ngtcp2_pkt_split_vec_at splits data[0] at offset |at|, and the
+ * right side of ngtcp2_vec is assigned to data[datacnt].  Similarly,
+ * offsets[0] + |at| is assigned to offsets[datacnt].  |data| must
+ * point to the array of ngtcp2_vec of length |datacnt|, and |datacnt|
+ * must be greater than 0.  |at| must be strictly less than data->len.
+ *
+ * This function returns |datacnt| + 1.
+ */
+size_t ngtcp2_pkt_split_vec_at(ngtcp2_vec *data, size_t datacnt,
+                               uint64_t *offsets, size_t at);
+
+/*
+ * ngtcp2_pkt_find_server_name searches TLS Server Name Indication
+ * extension in |v|.  If it is found, assign the portion of server
+ * name to the object pointed by |server_name|, and returns nonzero.
+ * Otherwise, it returns 0.  If |v| contains the extension partially,
+ * the function returns 0.  |v| must not be empty.
+ */
+int ngtcp2_pkt_find_server_name(ngtcp2_vec *server_name, const ngtcp2_vec *v);
+
+/*
+ * ngtcp2_pkt_append_ping_and_padding appends PING and PADDING frames
+ * to the array pointed by |data| of length |datacnt|.  The capacity
+ * of array must be at least NGTCP2_MAX_STREAM_DATACNT.  |n| is the
+ * number of bytes available for serialized PING and PADDING frames.
+ * |pcg| is a random number generator.  Which frames to add is
+ * determined randomly.
+ *
+ * The special encoding of PING and PADDING frames into ngtcp2_vec:
+ *
+ * - .base is NULL.
+ * - If .len is 0, it represents PING.  Otherwise, PADDING of .len
+ *   length.
+ *
+ * This function returns |datacnt| plus the number of frames added.
+ */
+size_t ngtcp2_pkt_append_ping_and_padding(ngtcp2_vec *data, size_t datacnt,
+                                          ngtcp2_pcg32 *pcg, size_t n);
+
+/*
+ * ngtcp2_pkt_permutate_vec permutates |data| and |offsets|, both have
+ * the |datacnt| elements.  |pcg| is a random number generator.
+ */
+void ngtcp2_pkt_permutate_vec(ngtcp2_vec *data, size_t datacnt,
+                              uint64_t *offsets, ngtcp2_pcg32 *pcg);
+
+/*
+ * ngtcp2_pkt_remove_vec_partial removes the portion of data that
+ * contains part of |part| from data[0].  This function does not
+ * remove whole range of |part|.  The length of removed data is chosen
+ * randomly.  The removed portion of data is assigned to the object
+ * pointed by |removed_data|.  If there is data located after the
+ * removed data, it will be assigned to data[datacnt].
+ * offsets[datacnt] is also updated, and the function returns
+ * |datacnt| + 1.  Otherwise, this function returns |datacnt|.
+ */
+size_t ngtcp2_pkt_remove_vec_partial(ngtcp2_vec *removed_data, ngtcp2_vec *data,
+                                     size_t datacnt, uint64_t *offsets,
+                                     ngtcp2_pcg32 *pcg, const ngtcp2_vec *part);
+
+#endif /* !defined(NGTCP2_PKT_H) */
