@@ -59,24 +59,41 @@
 #include "ngtcp2_addr_test.h"
 #include "ngtcp2_pcg_test.h"
 #include "ngtcp2_ratelim_test.h"
+#include "ngtcp2_conn_info_test.h"
 
 int main(int argc, char *argv[]) {
   const MunitSuite suites[] = {
-    pkt_suite,      range_suite,
-    rob_suite,      acktr_suite,
-    map_suite,      transport_params_suite,
-    rtb_suite,      idtr_suite,
-    conn_suite,     ringbuf_suite,
-    conv_suite,     ksl_suite,
-    gaptr_suite,    vec_suite,
-    strm_suite,     pv_suite,
-    pmtud_suite,    str_suite,
-    tstamp_suite,   cc_suite,
-    qlog_suite,     window_filter_suite,
-    settings_suite, callbacks_suite,
-    ppe_suite,      dcidtr_suite,
-    addr_suite,     pcg_suite,
-    ratelim_suite,  {0},
+    pkt_suite,
+    range_suite,
+    rob_suite,
+    acktr_suite,
+    map_suite,
+    transport_params_suite,
+    rtb_suite,
+    idtr_suite,
+    conn_suite,
+    ringbuf_suite,
+    conv_suite,
+    ksl_suite,
+    gaptr_suite,
+    vec_suite,
+    strm_suite,
+    pv_suite,
+    pmtud_suite,
+    str_suite,
+    tstamp_suite,
+    cc_suite,
+    qlog_suite,
+    window_filter_suite,
+    settings_suite,
+    callbacks_suite,
+    ppe_suite,
+    dcidtr_suite,
+    addr_suite,
+    pcg_suite,
+    ratelim_suite,
+    conn_info_suite,
+    {0},
   };
   const MunitSuite suite = {
     .prefix = "",
