@@ -27,7 +27,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif /* defined(HAVE_CONFIG_H) */
 
 #define MUNIT_ENABLE_ASSERT_ALIASES
 
@@ -35,6 +35,6 @@
 
 extern const MunitSuite idtr_suite;
 
-munit_void_test_decl(test_ngtcp2_idtr_open);
+munit_void_test_decl(test_ngtcp2_idtr_open)
 
-#endif /* NGTCP2_IDTR_TEST_H */
+#endif /* !defined(NGTCP2_IDTR_TEST_H) */
