@@ -27,10 +27,16 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif /* defined(HAVE_CONFIG_H) */
 
-void test_ngtcp2_range_intersect(void);
-void test_ngtcp2_range_cut(void);
-void test_ngtcp2_range_not_after(void);
+#define MUNIT_ENABLE_ASSERT_ALIASES
 
-#endif /* NGTCP2_RANGE_TEST_H */
+#include "munit.h"
+
+extern const MunitSuite range_suite;
+
+munit_void_test_decl(test_ngtcp2_range_intersect)
+munit_void_test_decl(test_ngtcp2_range_cut)
+munit_void_test_decl(test_ngtcp2_range_not_after)
+
+#endif /* !defined(NGTCP2_RANGE_TEST_H) */
