@@ -35,7 +35,7 @@
 
 extern const MunitSuite ringbuf_suite;
 
-munit_void_test_decl(test_ngtcp2_ringbuf_push_front);
-munit_void_test_decl(test_ngtcp2_ringbuf_pop_front);
+munit_void_test_decl(test_ngtcp2_ringbuf_push_front)
+munit_void_test_decl(test_ngtcp2_ringbuf_pop_front)
 
 #endif /* !defined(NGTCP2_RINGBUF_TEST_H) */
